// Spin.cpp : Implementation of CSpin
#include "stdafx.h"
#include <stdio.h>	// to use 'sprintf'

#include "SpinDlg.h"
#include "Spin.h"

/////////////////////////////////////////////////////////////////////////////
// CSpin

HWND CSpin::CreateControlWindow( HWND hWndParent, RECT& rcPos )
{
	if (CWindowImpl<CSpin>::Create(hWndParent,rcPos ))
	{	// Init, create and resize child dialog contained STATIC and UPDOWN windows
		m_dlg.m_spin = this;
		if (m_dlg.Create(m_hWnd))
		{
			RECT rc;
			GetClientRect(&rc);
			m_dlg.SetWindowPos(NULL,&rc,SWP_NOZORDER|SWP_SHOWWINDOW);
		}
		DWORD style = GetWindowLong(GWL_STYLE);
		style = style | WS_CLIPSIBLINGS |WS_CLIPCHILDREN;
		SetWindowLong(GWL_STYLE,style);
	}
	return m_hWnd;
}

LRESULT CSpin::OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if (m_dlg.m_hWnd)
		m_dlg.DestroyWindow();
	m_dlg.m_hWnd = 0;
	CWindowImpl<CSpin>::DestroyWindow();
	return 0;
}

LRESULT CSpin::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	WORD nWidth = LOWORD(lParam);
	WORD nHeight = HIWORD(lParam);
	if (nHeight && nWidth && m_dlg.m_hWnd)
		::SetWindowPos(m_dlg.m_hWnd,NULL,0,0,nWidth,nHeight,SWP_NOZORDER|SWP_NOMOVE);
	return 0;
}

HRESULT CSpin::OnDraw(ATL_DRAWINFO& di)
{
	// Draw something to output to printer..
	RECT& rc = *(RECT*)di.prcBounds;
	HBRUSH br = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
	FillRect(di.hdcDraw,&rc,br);
	DeleteObject(br);
	_TCHAR buf[48];
	_stprintf(buf,"%d",int(m_val.f));
	DrawText(di.hdcDraw,buf, -1, &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
	return S_OK;
}

void CSpin::GetSpinBounds(short& vmin,short& vmax)
{
	if (IsHex()==0)
	{
		vmin = short(m_data.v_min.f);
		vmax = short(m_data.v_max.f);
	}
	else
	{
		vmin = short(m_data.v_min.l);
		vmax = short(m_data.v_max.l);
	}
}

void CSpin::GetSpinPos(short& pos)
{
	if (IsHex()==0)
		pos = short(m_val.f);
	else
		pos = short(m_val.l);
}

BOOL CSpin::SetSpinPos(short new_pos)
{
	rt_chan_value old_val;
	old_val.v.f = m_val.f;
	m_val.f = new_pos;
	// Now try send..
	if (m_dataLink)
	{
		rt_chan_value cv;
		cv.chan = m_data.chi.chanId;
		cv.attr = m_data.chi.attrId;
		if (IsHex()==0)
			cv.v.f = m_val.f;
		else
			cv.v.l = m_val.f;
		HRESULT res = m_dataLink->ChangeChans(1,&cv);
		if (HRESULT_CODE(res)!=S_OK)
		{
			m_val.f = old_val.v.f;
			return FALSE;
		}
	}
	return TRUE;
}

STDMETHODIMP CSpin::GetSpinData(spin_data_t* d)
{
	if (NULL==d)
		return ResultFromScode(E_POINTER);
	d->v_min.f = m_data.v_min.f;
	d->v_max.f = m_data.v_max.f;
	d->flags = m_data.flags;
	return S_OK;
}

STDMETHODIMP CSpin::SetSpinData(spin_data_t* d)
{
	if (NULL==d)
		return ResultFromScode(E_POINTER);
	m_data.v_min.f = d->v_min.f;
	m_data.v_max.f = d->v_max.f;
	m_data.flags = d->flags;
	if (m_dlg.m_hWnd)
		m_dlg.Update();
	return S_OK;
}

////////////////// General IChInfo interface part //////////////////

STDMETHODIMP CSpin::SetDataLink(LPUNKNOWN dlink)
{
	if (dlink == NULL)
		return ResultFromScode(E_POINTER);
	if (m_dataLink)
		m_dataLink->Release();
	m_dataLink = (ItmDataLink*)dlink;
	m_dataLink->AddRef();
	return S_OK;
}

STDMETHODIMP CSpin::HasBuffer(BOOL* pVal)
{
	if (pVal == NULL)
		return ResultFromScode(E_POINTER);
	*pVal = FALSE;
	return S_OK;
}

// Return as soon as possible!
STDMETHODIMP CSpin::OnDataChanged(CHANINFO* chi,VALUEPOINT* vp,int) 
{ 
	if (!chi || !vp)
		return ResultFromScode(E_POINTER);
	if (chi->chanId==m_data.chi.chanId && chi->attrId==m_data.chi.attrId)
	{
		short pos = UD_MAXVAL;
		if (IsHex()==0)
		{
			if (vp->val<m_data.v_min.f)
				m_val.f = m_data.v_min.f;
			else if (vp->val>m_data.v_max.f)
				m_val.f = m_data.v_max.f;
			else
				m_val.f = vp->val;
			pos = short(m_val.f);
		}
		else
			m_val.f = pos = *(long*)(&vp->val);

		if (m_dlg.m_hWnd && pos != UD_MAXVAL)
			::SendMessage(m_dlg.m_hwSpin,UDM_SETPOS,0,(LPARAM) MAKELONG(pos,0));
	}
	return S_OK;
}


STDMETHODIMP CSpin::get_TrackApply(BOOL * pVal)
{
	if (pVal == NULL)
		return ResultFromScode(E_POINTER);
	*pVal = m_trackApply;
	return S_OK;
}

STDMETHODIMP CSpin::put_TrackApply(BOOL newVal)
{
	m_trackApply = newVal;
	return S_OK;
}

STDMETHODIMP CSpin::get_UseArchive(BOOL * pVal)
{
	if (pVal == NULL)
		return ResultFromScode(E_POINTER);
	*pVal = FALSE;
	return S_OK;
}

STDMETHODIMP CSpin::put_UseArchive(BOOL)
{
	return S_FALSE;
}

STDMETHODIMP CSpin::SetChanBrowser(LPUNKNOWN u)
{
	if (NULL==u)
		return ResultFromScode(E_POINTER);
	if (m_chb)
		m_chb->Release();
	m_chb = (ItmChBrowser*)u;
	m_chb->AddRef();
	return S_OK;
}

STDMETHODIMP CSpin::GetChanBrowser(LPUNKNOWN* dest)
{
	if (dest == NULL)
		return ResultFromScode(E_POINTER);
	*dest = m_chb;
	return S_OK;
}

STDMETHODIMP CSpin::get_IsVisibleAtStart(BOOL *pVal)
{
	if (NULL==pVal)
		return ResultFromScode(E_POINTER);
	*pVal = (m_data.flags & TMX_VISIBLEATSTART)!=0;
	return S_OK;
}

STDMETHODIMP CSpin::GetSizeMax(ULARGE_INTEGER FAR* pcbSize)
{
	if (NULL==pcbSize)
		return ResultFromScode(E_POINTER);
	ULISet32(*pcbSize,sizeof(spin_data_t));
	return S_OK;
}

STDMETHODIMP CSpin::Load(LPSTREAM pStm)
{
	if (NULL==pStm)
		return ResultFromScode(E_POINTER);
	return pStm->Read(&m_data,sizeof(spin_data_t),0);
}

STDMETHODIMP CSpin::Save( LPSTREAM pStm, BOOL fClearDirty )
{
	if (NULL==pStm)
		return ResultFromScode(E_POINTER);
	return pStm->Write(&m_data,sizeof(spin_data_t),0);
}

