// Spin.h : Declaration of the CSpin

#ifndef __SPIN_H_
#define __SPIN_H_

#include "SpinDlg.h"
#include "Spinpage.h"

#include "resource.h"       // main symbols

#import "chbase.tlb"
using namespace CHBASELib;

/////////////////////////////////////////////////////////////////////////////
// CSpin
class ATL_NO_VTABLE CSpin : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CSpin, &CLSID_Spin>,
	public CComControl<CSpin>,
	public ItmSpin,
	public IChInfo,
	public IProvideClassInfo2Impl<&CLSID_Spin, NULL, &LIBID_TMXLIBLib>,
	public IPersistStreamInitImpl<CSpin>,
	public IPersistStorageImpl<CSpin>,
	public IQuickActivateImpl<CSpin>,
	public IOleControlImpl<CSpin>,
	public IOleObjectImpl<CSpin>,
	public IOleInPlaceActiveObjectImpl<CSpin>,
	public IViewObjectExImpl<CSpin>,
	public IOleInPlaceObjectWindowlessImpl<CSpin>,
	public IDataObjectImpl<CSpin>,
	public ISpecifyPropertyPagesImpl<CSpin>
{
public:
	CSpin()
	{
		m_bWindowOnly = TRUE; // In visible state the control windowed always
		m_dataLink = NULL;		// Will be set by TraceMode
		m_chb = NULL;			// ---//---
		memset(&m_data,0,sizeof(m_data));
		m_data.v_max.f = 100;
		m_data.flags = TMX_MINMAXFROMSCALE|TMX_VISIBLEATSTART;
		m_val.f = 0;
	}
	~CSpin()
	{
		if (m_dataLink)
			m_dataLink->Release();
		if (m_chb)
			m_chb->Release();
	}
	CSpinDlg m_dlg;

	ItmDataLink *m_dataLink;
	ItmChBrowser *m_chb;
	spin_data_t m_data;
	rt_value m_val;

	BOOL m_trackApply;

DECLARE_REGISTRY_RESOURCEID(IDR_SPIN)

BEGIN_COM_MAP(CSpin)
	COM_INTERFACE_ENTRY(ItmSpin)
	COM_INTERFACE_ENTRY(IChInfo)
	COM_INTERFACE_ENTRY_IMPL(IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY_IMPL(IOleControl)
	COM_INTERFACE_ENTRY_IMPL(IOleObject)
	COM_INTERFACE_ENTRY_IMPL(IQuickActivate)
	COM_INTERFACE_ENTRY_IMPL(IPersistStorage)
	COM_INTERFACE_ENTRY_IMPL(IPersistStreamInit)
	COM_INTERFACE_ENTRY_IMPL(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY_IMPL(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROPERTY_MAP(CSpin)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	//PROP_PAGE(CLSID_StockColorPage)

	PROP_PAGE(CLSID_SpinPage)
	PROP_PAGE(__uuidof(tmChanPage))
END_PROPERTY_MAP()


BEGIN_MSG_MAP(CSpin)
	MESSAGE_HANDLER(WM_PAINT, OnPaint)
	MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
	MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
	MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
	MESSAGE_HANDLER(WM_SIZE, OnSize)
	MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBk)
END_MSG_MAP()


// IViewObjectEx
	STDMETHOD(GetViewStatus)(DWORD* pdwStatus)
	{
		ATLTRACE(_T("IViewObjectExImpl::GetViewStatus\n"));
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}
	BOOL IsHex() { return ((m_data.flags & TMX_HEX)!=0); }
	void GetSpinBounds(short&,short&);
	void GetSpinPos(short&);
	BOOL SetSpinPos(short);

// ItmSpin
public:
	HRESULT OnDraw(ATL_DRAWINFO& di);
	HWND CreateControlWindow( HWND hWndParent, RECT& rcPos );

	STDMETHOD(SetChInfo)(CHANINFO* _chi) 
	{ 
		if (_chi)
		{
			memcpy(&m_data.chi,_chi,sizeof(CHANINFO)); 
			if (m_hWnd)
				Invalidate(0);
		}
		return S_OK;
	}
	STDMETHOD(GetChInfo)(CHANINFO* _chi){ if (_chi) memcpy(_chi,&m_data.chi,sizeof(CHANINFO)); return S_OK;}
	STDMETHOD(SetDataLink)(LPUNKNOWN);
	STDMETHOD(SetArchSource)(LPUNKNOWN) { return S_FALSE;}

	// Channel binding page will be used if GetChInfoEx() not return S_OK
	STDMETHOD(GetChInfoEx)(long*,CHANINFO**) { return ResultFromScode(E_NOTIMPL); }

	STDMETHOD(HasBuffer)(BOOL*);
	STDMETHOD(OnDataChanged)(CHANINFO*,VALUEPOINT*,int);
	STDMETHOD(get_TrackApply)(BOOL *pVal);
	STDMETHOD(put_TrackApply)(BOOL newVal);
	STDMETHOD(get_IsVisibleAtStart)(BOOL *pVal);
	STDMETHOD(get_UseArchive)(BOOL *pVal);
	STDMETHOD(put_UseArchive)(BOOL newVal);
	STDMETHOD(SetArchTime)(unsigned long sec,unsigned short msec) { return S_FALSE; }
	STDMETHOD(SetChanBrowser)(LPUNKNOWN u);
	STDMETHOD(GetChanBrowser)(LPUNKNOWN* dest);
							  
	STDMETHOD(NeedAlarm)(BOOL *pVal) { if (!pVal) return ResultFromScode(E_POINTER); 
				else {*pVal = FALSE; return S_OK; } }
	STDMETHOD(OnAlarm)(wchar_t *alarm)	{ return ResultFromScode(E_NOTIMPL); }
	STDMETHOD(OnRun)(BOOL r) { return S_OK; }

// IPersistStreamInitImpl - must be implemented!!!
	STDMETHOD(GetClassID)(CLSID *pClassID)
	{
		*pClassID = CLSID_Spin;
		return S_OK;
	}
	STDMETHOD(GetSizeMax)(ULARGE_INTEGER FAR* pcbSize);
	STDMETHOD(Load)(LPSTREAM pStm);
	STDMETHOD(Save)( LPSTREAM pStm, BOOL fClearDirty );
	STDMETHOD(IsDirty)(void) { return S_OK; }

	STDMETHOD(GetFlags)(unsigned long *pVal) { return S_FALSE; }

	STDMETHOD(GetSpinData)(spin_data_t* d);
	STDMETHOD(SetSpinData)(spin_data_t* d);
	LRESULT OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnEraseBk(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return 0;
	}
};

#endif //__SPIN_H_
