unit TmDelCtlImpl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActiveX, AxCtrls, TmDelCtlProj_TLB,TMRtmData_TLB,StdVcl, StdCtrls;

// Groups codes
const RG_OBJECTS: Longint = 1;
const RG_CHANNELS: Longint = 2;
const RG_CHAN_ATTRS: Longint = 3;
const RG_CHAN_ARCH_ATTRS: Longint = 4;
const RG_CAT_ATTRS: Longint = 5;
// Objects fields codes
const RAO_ID: Longint = $40000;
const RAO_NAME: Longint = $40001;
const RAO_CHAN_CNT: Longint = $40002;
// Channels fields codes
const RAC_ID: Longint = $100;
const RAC_NAME: Longint = $101;
const RAC_UNITS: Longint = $102;
const RAC_PERIOD: Longint = $103; // reserved
const RAC_TYPE: Longint = $104; // 0-analog 1-discrete
const RAC_NBITS: Longint = $105; // for discretes only
const RAC_ATTR_CNT: Longint = $106;
const RAC_CODE: Longint = $200;
const RAC_USIGNAL: Longint = $300; // reserved
// Attributes fields codes
const RAA_ID: Longint = $50000;
const RAA_NAME: Longint = $50001;
// Values quality
const RQ_BAD: Longint = $0;
const RQ_UNCERTAIN: Longint = $40;
const RQ_GOOD: Longint = $C0;
// TMX flags
const TF_BUFFERED: Longint = $1;
const TF_NEEDALARM: Longint = $2;
const TF_HIDEONRUN: Longint = $4;

type
  TTmDelCtl = class(TActiveForm, ITmDelCtl, IDtmRtmClient)
    ObjsCombo: TComboBox;
    ChansCombo: TComboBox;
    AttrsCombo: TComboBox;
    ChanId: TLabel;
    ChanUnits: TLabel;
    ChanType: TLabel;
    ChanEncoding: TLabel;
    ChanNBits: TLabel;
    InpTags: TListBox;
    AddInp: TButton;
    OutpTags: TListBox;
    AddOutp: TButton;
    RemOutp: TButton;
    RemInp: TButton;
    ToSend: TEdit;
    Send: TButton;
    Alarm: TLabel;
    HideOnRun: TCheckBox;
    NeedAlarm: TCheckBox;
    Refresh: TButton;
    Label1: TLabel;
    procedure OnObjChanged(Sender: TObject);
    procedure OnChanChanged(Sender: TObject);
    procedure OnAttrChanged(Sender: TObject);
    procedure OnInpAdd(Sender: TObject);
    procedure OnAddOutp(Sender: TObject);
    procedure OnRemOutp(Sender: TObject);
    procedure OnRemInp(Sender: TObject);
    procedure OnSend(Sender: TObject);
    procedure OnValClick(Sender: TObject);
    procedure OnNeedAlarmClick(Sender: TObject);
    procedure OnStartHideClick(Sender: TObject);
    procedure OnRefreshClick(Sender: TObject);
  private
    { Private declarations }
    FEvents: ITmDelCtlEvents;
    m_tmserver: IDtmRtmData;
    n_objs: Longint;
    n_chans: Longint;
    n_attrs: Longint;
    objs_ids: array of Longint;
    chans_ids: array of Longint;
    attrs_ids: array of Longint;
    // temporary
    dat: TDateTime;
    prims: array of Longint;
    secs: array of Longint;
    vals: array of Variant;
    // Input tags
    in_chans: array of Longint;
    in_attrs: array of Longint;
    in_vals: array of Variant;
    in_tags_count: Longint;
    // Output tags
    o_chans: array of Longint;
    o_attrs: array of Longint;
    o_vals: array of Variant;
    o_tags_count: Longint;
    status : Smallint;
    //statuses: array of Integer;
    tfs:Longint;
    run:Boolean;

    procedure ActivateEvent(Sender: TObject);
    procedure ClickEvent(Sender: TObject);
    procedure CreateEvent(Sender: TObject);
    procedure DblClickEvent(Sender: TObject);
    procedure DeactivateEvent(Sender: TObject);
    procedure DestroyEvent(Sender: TObject);
    procedure KeyPressEvent(Sender: TObject; var Key: Char);
    procedure PaintEvent(Sender: TObject);

    procedure setServer(host: OleVariant); safecall;
    procedure getBindings(var n: Integer; var flags: Integer); safecall;
    procedure readTags(count: Integer; var prim_ids: OleVariant; var sec_ids: OleVariant); safecall;
    procedure writeTags(count: Integer; prim_ids: OleVariant; sec_ids: OleVariant); safecall;
    procedure onAlarm(const a: WideString); safecall;
    procedure onRun(r: Integer); safecall;
    procedure onTagsValsChanged(count: Integer; prim_ids: OleVariant; sec_ids: OleVariant;
                                val_attrs: OleVariant; vals: OleVariant; var ts: TDateTime); safecall;
    procedure getValsBufSize(chan: Integer; attr: Integer; var count: Integer); safecall;
    procedure readValsBuffer(count: Integer; chan: Integer; attr: Integer; var vals: OleVariant;
                             var timestamps: OleVariant); safecall;
    procedure writeValsBuffer(count: Integer; chan: Integer; attr: Integer; vals: OleVariant;
                              timestamps: OleVariant); safecall;

    procedure RefillObjs();
    procedure RefillChans();
    procedure RefillAttrs();
    procedure RemoveOutpTag(index:Longint);
    procedure RemoveInpTag(index:Longint);
    procedure AddInpTag(chan:Longint; attr:Longint);
    procedure AddOutpTag(chan:Longint; attr:Longint);
    procedure UpdateInpVal(i_tag:Longint; val:Variant);
  protected
    { Protected declarations }
    procedure DefinePropertyPages(DefinePropertyPage: TDefinePropertyPage); override;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_Active: WordBool; safecall;
    function Get_AutoScroll: WordBool; safecall;
    function Get_AutoSize: WordBool; safecall;
    function Get_AxBorderStyle: TxActiveFormBorderStyle; safecall;
    function Get_Caption: WideString; safecall;
    function Get_Color: OLE_COLOR; safecall;
    function Get_Cursor: Smallint; safecall;
    function Get_DoubleBuffered: WordBool; safecall;
    function Get_DropTarget: WordBool; safecall;
    function Get_Enabled: WordBool; safecall;
    function Get_Font: IFontDisp; safecall;
    function Get_HelpFile: WideString; safecall;
    function Get_KeyPreview: WordBool; safecall;
    function Get_PixelsPerInch: Integer; safecall;
    function Get_PrintScale: TxPrintScale; safecall;
    function Get_Scaled: WordBool; safecall;
    function Get_Visible: WordBool; safecall;
    function Get_VisibleDockClientCount: Integer; safecall;
    procedure _Set_Font(const Value: IFontDisp); safecall;
    procedure Set_AutoScroll(Value: WordBool); safecall;
    procedure Set_AutoSize(Value: WordBool); safecall;
    procedure Set_AxBorderStyle(Value: TxActiveFormBorderStyle); safecall;
    procedure Set_Caption(const Value: WideString); safecall;
    procedure Set_Color(Value: OLE_COLOR); safecall;
    procedure Set_Cursor(Value: Smallint); safecall;
    procedure Set_DoubleBuffered(Value: WordBool); safecall;
    procedure Set_DropTarget(Value: WordBool); safecall;
    procedure Set_Enabled(Value: WordBool); safecall;
    procedure Set_Font(var Value: IFontDisp); safecall;
    procedure Set_HelpFile(const Value: WideString); safecall;
    procedure Set_KeyPreview(Value: WordBool); safecall;
    procedure Set_PixelsPerInch(Value: Integer); safecall;
    procedure Set_PrintScale(Value: TxPrintScale); safecall;
    procedure Set_Scaled(Value: WordBool); safecall;
    procedure Set_Visible(Value: WordBool); safecall;
    function Get_TmxStyle: Integer; safecall;
    procedure Set_TmxStyle(Value: Integer); safecall;
  public
    { Public declarations }
    procedure Initialize; override;

    procedure LoadCompProperty(Reader: TReader);
    procedure StoreCompProperty(Writer: TWriter);
    procedure DefineProperties(Filer: TFiler); override;
  end;

implementation

uses ComObj, ComServ;

{$R *.DFM}

{ TTmDelCtl }

procedure TTmDelCtl.DefinePropertyPages(DefinePropertyPage: TDefinePropertyPage);
begin
  { Define property pages here.  Property pages are defined by calling
    DefinePropertyPage with the class id of the page.  For example,
      DefinePropertyPage(Class_TmDelCtlPage); }
end;

procedure TTmDelCtl.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as ITmDelCtlEvents;
end;

procedure TTmDelCtl.Initialize;
begin
  inherited Initialize;
  OnActivate := ActivateEvent;
  OnClick := ClickEvent;
  OnCreate := CreateEvent;
  OnDblClick := DblClickEvent;
  OnDeactivate := DeactivateEvent;
  OnDestroy := DestroyEvent;
  OnKeyPress := KeyPressEvent;
  OnPaint := PaintEvent;
  m_tmserver := nil;
  tfs := 0;
  Alarm.Visible := false;
end;

procedure TTmDelCtl.LoadCompProperty(Reader: TReader);
begin
    tfs := Reader.ReadInteger;
    if (tfs and TF_NEEDALARM)<>0 then
        NeedAlarm.Checked := true;
    if (tfs and TF_HIDEONRUN)<>0 then
        HideOnRun.Checked := true;
    Alarm.Visible := NeedAlarm.Checked;
end;

procedure TTmDelCtl.StoreCompProperty(Writer: TWriter);
begin
    Writer.WriteInteger(tfs);
end;

procedure TTmDelCtl.DefineProperties(Filer: TFiler);
begin
  inherited; { allow base classes to define properties }
  Filer.DefineProperty('TmxStyle', LoadCompProperty, StoreCompProperty, true);
end;

function TTmDelCtl.Get_Active: WordBool;
begin
  Result := Active;
end;

function TTmDelCtl.Get_AutoScroll: WordBool;
begin
  Result := AutoScroll;
end;

function TTmDelCtl.Get_AutoSize: WordBool;
begin
  Result := AutoSize;
end;

function TTmDelCtl.Get_AxBorderStyle: TxActiveFormBorderStyle;
begin
  Result := Ord(AxBorderStyle);
end;

function TTmDelCtl.Get_Caption: WideString;
begin
  Result := WideString(Caption);
end;

function TTmDelCtl.Get_Color: OLE_COLOR;
begin
  Result := OLE_COLOR(Color);
end;

function TTmDelCtl.Get_Cursor: Smallint;
begin
  Result := Smallint(Cursor);
end;

function TTmDelCtl.Get_DoubleBuffered: WordBool;
begin
  Result := DoubleBuffered;
end;

function TTmDelCtl.Get_DropTarget: WordBool;
begin
  Result := DropTarget;
end;

function TTmDelCtl.Get_Enabled: WordBool;
begin
  Result := Enabled;
end;

function TTmDelCtl.Get_Font: IFontDisp;
begin
  GetOleFont(Font, Result);
end;

function TTmDelCtl.Get_HelpFile: WideString;
begin
  Result := WideString(HelpFile);
end;

function TTmDelCtl.Get_KeyPreview: WordBool;
begin
  Result := KeyPreview;
end;

function TTmDelCtl.Get_PixelsPerInch: Integer;
begin
  Result := PixelsPerInch;
end;

function TTmDelCtl.Get_PrintScale: TxPrintScale;
begin
  Result := Ord(PrintScale);
end;

function TTmDelCtl.Get_Scaled: WordBool;
begin
  Result := Scaled;
end;

function TTmDelCtl.Get_Visible: WordBool;
begin
  Result := Visible;
end;

function TTmDelCtl.Get_VisibleDockClientCount: Integer;
begin
  Result := VisibleDockClientCount;
end;

procedure TTmDelCtl._Set_Font(const Value: IFontDisp);
begin
  SetOleFont(Font, Value);
end;

procedure TTmDelCtl.ActivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnActivate;
end;

procedure TTmDelCtl.ClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnClick;
end;

procedure TTmDelCtl.CreateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnCreate;
end;

procedure TTmDelCtl.DblClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDblClick;
end;

procedure TTmDelCtl.DeactivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDeactivate;
end;

procedure TTmDelCtl.DestroyEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDestroy;
end;

procedure TTmDelCtl.KeyPressEvent(Sender: TObject; var Key: Char);
var
  TempKey: Smallint;
begin
  TempKey := Smallint(Key);
  if FEvents <> nil then FEvents.OnKeyPress(TempKey);
  Key := Char(TempKey);
end;

procedure TTmDelCtl.PaintEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnPaint;
end;

procedure TTmDelCtl.Set_AutoScroll(Value: WordBool);
begin
  AutoScroll := Value;
end;

procedure TTmDelCtl.Set_AutoSize(Value: WordBool);
begin
  AutoSize := Value;
end;

procedure TTmDelCtl.Set_AxBorderStyle(Value: TxActiveFormBorderStyle);
begin
  AxBorderStyle := TActiveFormBorderStyle(Value);
end;

procedure TTmDelCtl.Set_Caption(const Value: WideString);
begin
  Caption := TCaption(Value);
end;

procedure TTmDelCtl.Set_Color(Value: OLE_COLOR);
begin
  Color := TColor(Value);
end;

procedure TTmDelCtl.Set_Cursor(Value: Smallint);
begin
  Cursor := TCursor(Value);
end;

procedure TTmDelCtl.Set_DoubleBuffered(Value: WordBool);
begin
  DoubleBuffered := Value;
end;

procedure TTmDelCtl.Set_DropTarget(Value: WordBool);
begin
  DropTarget := Value;
end;

procedure TTmDelCtl.Set_Enabled(Value: WordBool);
begin
  Enabled := Value;
end;

procedure TTmDelCtl.Set_Font(var Value: IFontDisp);
begin
  SetOleFont(Font, Value);
end;

procedure TTmDelCtl.Set_HelpFile(const Value: WideString);
begin
  HelpFile := String(Value);
end;

procedure TTmDelCtl.Set_KeyPreview(Value: WordBool);
begin
  KeyPreview := Value;
end;

procedure TTmDelCtl.Set_PixelsPerInch(Value: Integer);
begin
  PixelsPerInch := Value;
end;

procedure TTmDelCtl.Set_PrintScale(Value: TxPrintScale);
begin
  PrintScale := TPrintScale(Value);
end;

procedure TTmDelCtl.Set_Scaled(Value: WordBool);
begin
  Scaled := Value;
end;

procedure TTmDelCtl.Set_Visible(Value: WordBool);
begin
  Visible := Value;
end;

procedure TTmDelCtl.RefillObjs();
    var i:Integer;
    var t0: OleVariant;
    var t1: OleVariant;
    var t2: OleVariant;
    var t3: OleVariant;
begin
    if m_tmserver<>nil then
    begin
        m_tmserver.getObjectCount(RG_OBJECTS,0,n_objs);
        if n_objs<>0 then
        begin
            SetLength(objs_ids,n_objs);
            t0 := objs_ids;
            m_tmserver.listObjects(n_objs, RG_OBJECTS, 0, t0);
            ObjsCombo.Clear;
            SetLength(secs,n_objs);
            SetLength(vals,n_objs);
            for i:=0 to n_objs do
                secs[i] := RAO_NAME;
            t1 := secs;
            t2 := vals;
            VarClear(t3);
            m_tmserver.readValues(n_objs, t0, t1, t3, t2, dat);
            for i:=0 to n_objs-1 do
            begin
                ObjsCombo.Items.Add(t2[i]);
                objs_ids[i] := t0[i];
            end;
            secs := nil;
        end;
    end;
end;

procedure TTmDelCtl.RefillChans();
    var i:Integer;
    var t0: OleVariant;
    var t1: OleVariant;
    var t2: OleVariant;
    var t3: OleVariant;
    var cur_obj: Longint;
begin
    if m_tmserver<>nil then
    begin
        i := ObjsCombo.ItemIndex;
        if i>=0 then
        begin
            cur_obj := objs_ids[i];
            prims := nil;
            SetLength(prims,1);
            t2 := vals;
            VarArrayRedim(t2,1);
            SetLength(secs,1);
            prims[0] := cur_obj;
            secs[0] := RAO_CHAN_CNT;
            vals[0] := 0;
            t0 := prims;
            t1 := secs;
            VarClear(t3);
            m_tmserver.readValues(1, t0, t1, t3, t2, dat);
            n_chans := t2[0];
            ChansCombo.Clear;
            if n_chans>0 then
            begin
                chans_ids := nil;
                SetLength(chans_ids,n_chans);
                t0 := chans_ids;
                m_tmserver.listObjects(n_chans, RG_CHANNELS,cur_obj, t0);
                prims := nil;
                secs := nil;
                SetLength(secs,n_chans);
                VarArrayRedim(t2,n_chans);
                for i := 0 to n_chans - 1 do
                begin
                    secs[i] := RAC_NAME;
                    chans_ids[i] := t0[i];
                end;
                t0 := chans_ids;
                t1 := secs;
                VarClear(t3);
                m_tmserver.readValues(n_chans, t0, t1, t3, t2, dat);
                for i := 0 to n_chans - 1 do
                begin
                    ChansCombo.Items.Add(t2[i]);
                    chans_ids[i] := t0[i];
                end;
                prims := nil;
                secs := nil;
            end;
        end;
    end;
end;

procedure TTmDelCtl.RefillAttrs();
    var i:Integer;
    var t0: OleVariant;
    var t1: OleVariant;
    var t2: OleVariant;
    var t3: OleVariant;
    var cur_chan: Longint;
begin
    if m_tmserver<>nil then
    begin
        i := ChansCombo.ItemIndex;
        if i>=0 then
        begin
            cur_chan := chans_ids[i];
            prims := nil;
            SetLength(prims,1);
            t2 := vals;
            VarArrayRedim(t2,1);
            SetLength(secs,1);
            prims[0] := cur_chan;
            secs[0] := RAC_ATTR_CNT;
            vals[0] := 0;
            t0 := prims;
            t1 := secs;
            VarClear(t3);
            m_tmserver.readValues(1, t0, t1, t3, t2, dat);
            n_attrs := t2[0];
            AttrsCombo.Clear;
            if n_attrs>0 then
            begin
                attrs_ids := nil;
                SetLength(attrs_ids,n_attrs);
                t0 := attrs_ids;
                m_tmserver.listObjects(n_attrs, RG_CHAN_ATTRS,cur_chan, t0);
                prims := nil;
                secs := nil;
                SetLength(secs,n_attrs);
                VarArrayRedim(t2,n_attrs);
                for i := 0 to n_attrs - 1 do
                begin
                    secs[i] := RAA_NAME;
                    attrs_ids[i] := t0[i];
                end;
                t0 := attrs_ids;
                t1 := secs;
                VarClear(t3);
                m_tmserver.readValues(n_attrs, t0, t1, t3, t2, dat);
                for i := 0 to n_attrs - 1 do
                    AttrsCombo.Items.Add(t2[i]);
                prims := nil;
                secs := nil;
            end;
        end;
    end;
end;

(* IDtmRtmClient members *)
procedure TTmDelCtl.setServer(host: OleVariant);
var t_serv: IDispatch;
begin
    t_serv := host;
    if t_serv<>nil then
    begin
        m_tmserver := t_serv as IDtmRtmData;
        RefillObjs();
    end
    else
        m_tmserver := nil;
    t_serv := nil;
end;

procedure TTmDelCtl.getBindings(var n: Integer; var flags: Integer);
begin
    n := in_tags_count;
    flags := tfs;
end;

procedure TTmDelCtl.readTags(count: Integer; var prim_ids: OleVariant; var sec_ids: OleVariant);
var i: Longint;
begin
    for i := 0 To in_tags_count - 1 do
    begin
        prim_ids[i] := in_chans[i];
        sec_ids[i] := in_attrs[i];
    end;
end;

procedure TTmDelCtl.writeTags(count: Integer; prim_ids: OleVariant; sec_ids: OleVariant);
var i: Longint;
begin
    for i := 0 To in_tags_count - 1 do
    begin
        in_chans[i] := prim_ids[i];
        in_attrs[i] := sec_ids[i];
    end;
end;

procedure TTmDelCtl.onAlarm(const a: WideString);
begin
    if (tfs and TF_NEEDALARM)<>0 then
        Alarm.Caption := a
    else
        Alarm.Caption := '';
end;

procedure TTmDelCtl.onRun(r: Integer);
begin
    run := (r <> 0);
end;

procedure TTmDelCtl.onTagsValsChanged(count: Integer; prim_ids: OleVariant; sec_ids: OleVariant;
                                val_attrs: OleVariant; vals: OleVariant; var ts: TDateTime);
var i_v,i_tag: Longint;
    t:Variant;
begin
    for i_tag := 0 to in_tags_count - 1 do
    begin
        for i_v := 0 To count - 1 do
            if (prim_ids[i_v] = in_chans[i_tag]) and (sec_ids[i_v] = in_attrs[i_tag]) then
            begin
                t := vals[i_v];
                in_vals[i_tag] := t;
                UpdateInpVal(i_tag, t);
            end;
    end;
end;

procedure TTmDelCtl.getValsBufSize(chan: Integer; attr: Integer; var count: Integer);
begin
    count := 0;
end;

procedure TTmDelCtl.readValsBuffer(count: Integer; chan: Integer; attr: Integer; var vals: OleVariant;
                             var timestamps: OleVariant);
begin
end;

procedure TTmDelCtl.writeValsBuffer(count: Integer; chan: Integer; attr: Integer; vals: OleVariant;
                              timestamps: OleVariant);
begin
end;

procedure TTmDelCtl.OnObjChanged(Sender: TObject);
begin
    RefillChans();
end;

procedure TTmDelCtl.OnChanChanged(Sender: TObject);
begin
    RefillAttrs();
end;

procedure TTmDelCtl.OnAttrChanged(Sender: TObject);
    var i_attr:Integer;
    var i_chan:Integer;
    var t0: OleVariant;
    var t1: OleVariant;
    var t2: OleVariant;
    var t3: OleVariant;
    var s: String;
//    var cur_chan: Longint;
begin
    if n_chans > 0 then
    begin
        i_chan := ChansCombo.ItemIndex;
        if i_chan >= 0 then
        begin
            Str(chans_ids[i_chan],s);
            ChanId.Caption := s;
            prims := nil;
            secs := nil;
//            t0 := prims;
//            t1 := secs;
            t2 := vals;
            SetLength(prims,1);
            SetLength(secs,1);
            VarArrayRedim(t2,1);
            prims[0] := chans_ids[i_chan];
            secs[0] := RAC_UNITS;
            t0 := prims;
            t1 := secs;
            t2 := vals;
            VarClear(t3);
            m_tmserver.readValues(1, t0, t1, t3, t2, dat);
            ChanUnits.Caption := t2[0];
            i_attr := AttrsCombo.ItemIndex;
            if i_attr >= 0 then
            begin
                prims[0] := chans_ids[i_chan];
                secs[0] := RAC_CODE + attrs_ids[i_attr];
                t0 := prims;
                t1 := secs;
                t2 := vals;
                VarClear(t3);
                m_tmserver.readValues(1, t0, t1, t3, t2, dat);
                ChanEncoding.Caption := t2[0];
            end;
            prims[0] := chans_ids[i_chan];
            secs[0] := RAC_TYPE;
            t0 := prims;
            t1 := secs;
            t2 := vals;
            VarClear(t3);
            m_tmserver.readValues(1, t0, t1, t3, t2, dat);
            ChanType.Caption := t2[0];
            prims[0] := chans_ids[i_chan];
            secs[0] := RAC_NBITS;
            t0 := prims;
            t1 := secs;
            t2 := vals;
            VarClear(t3);
            m_tmserver.readValues(1, t0, t1, t3, t2, dat);
            ChanNBits.Caption := t2[0];
        end;
    end;
end;

procedure TTmDelCtl.AddInpTag(chan:Longint; attr:Longint);
begin
    SetLength(in_chans,in_tags_count + 1);
    SetLength(in_attrs,in_tags_count + 1);
    SetLength(in_vals,in_tags_count + 1);
    in_chans[in_tags_count] := chan;
    in_attrs[in_tags_count] := attr;
    in_tags_count := in_tags_count + 1;
end;

procedure TTmDelCtl.AddOutpTag(chan:Longint; attr:Longint);
var t: OleVariant;
begin
    SetLength(o_chans,o_tags_count + 1);
    SetLength(o_attrs,o_tags_count + 1);
    SetLength(o_vals,o_tags_count + 1);
    t := o_vals;
    VarArrayRedim(t,o_tags_count + 1);
    o_chans[o_tags_count] := chan;
    o_attrs[o_tags_count] := attr;
    o_tags_count := o_tags_count + 1;
end;

procedure TTmDelCtl.RemoveInpTag(index:Longint);
var
    tmp_chan: array of Longint;
    tmp_attr: array of Longint;
    i,j,d: Longint;
begin
    if in_tags_count > 0 then
    begin
        SetLength(tmp_chan,in_tags_count);
        SetLength(tmp_attr,in_tags_count);
        for i := 0 To in_tags_count - 1 do
        begin
            tmp_chan[i] := in_chans[i];
            tmp_attr[i] := in_attrs[i];
        end;
        if in_tags_count>1 then
        begin
            SetLength(in_chans,in_tags_count-1);
            SetLength(in_chans,in_tags_count-1);
            SetLength(in_vals,in_tags_count-1);
        end;
        j := 0;
        d := 0;
        for i := 0 to in_tags_count - 1 do
        begin
            if i = index then
                d := 1;
            in_chans[j] := tmp_chan[i+d];
            in_attrs[j] := tmp_attr[i+d];
            j := j + 1;
        end;
        in_tags_count := in_tags_count - 1;
    end;
end;

procedure TTmDelCtl.RemoveOutpTag(index:Longint);
var
    tmp_chan: array of Longint;
    tmp_attr: array of Longint;
    i,j,d: Longint;
begin
    if o_tags_count > 0 then
    begin
        SetLength(tmp_chan,o_tags_count);
        SetLength(tmp_attr,o_tags_count);
        for i := 0 To o_tags_count - 1 do
        begin
            tmp_chan[i] := o_chans[i];
            tmp_attr[i] := o_attrs[i];
        end;
        if o_tags_count>1 then
        begin
            SetLength(o_chans,o_tags_count-1);
            SetLength(o_attrs,o_tags_count-1);
            SetLength(o_vals,o_tags_count-1);
        end;
        j := 0;
        d := 0;
        for i := 0 to o_tags_count - 1 do
        begin
            if i = index then
                d := 1;
            o_chans[j] := tmp_chan[i+d];
            o_attrs[j] := tmp_attr[i+d];
            j := j + 1;
        end;
        o_tags_count := o_tags_count - 1;
    end;
end;

procedure TTmDelCtl.OnInpAdd(Sender: TObject);
var i_chan,i_attr:Integer;
begin
    i_chan := ChansCombo.ItemIndex;
    i_attr := AttrsCombo.ItemIndex;
    if (i_chan>=0) and (i_attr>=0) then
    begin
        AddInpTag(chans_ids[i_chan],attrs_ids[i_attr]);
        InpTags.Items.Add(ChansCombo.Items[i_chan]+'.'+AttrsCombo.Items[i_attr]+' =');
    end;
end;

procedure TTmDelCtl.OnAddOutp(Sender: TObject);
var i_chan,i_attr:Integer;
begin
    i_chan := ChansCombo.ItemIndex;
    i_attr := AttrsCombo.ItemIndex;
    if (i_chan>=0) and (i_attr>=0) then
    begin
        AddOutpTag(chans_ids[i_chan],attrs_ids[i_attr]);
        OutpTags.Items.Add(ChansCombo.Items[i_chan]+'.'+AttrsCombo.Items[i_attr]);
    end;
end;

procedure TTmDelCtl.OnRemOutp(Sender: TObject);
var i:Longint;
begin
    i := OutpTags.ItemIndex;
    if i >= 0 then
    begin
        OutpTags.Items.Delete(i);
        RemoveOutpTag(i);
    end;
end;

procedure TTmDelCtl.OnRemInp(Sender: TObject);
var i:Longint;
begin
    i := InpTags.ItemIndex;
    if i >= 0 then
    begin
        InpTags.Items.Delete(i);
        RemoveInpTag(i);
    end;
end;

procedure TTmDelCtl.UpdateInpVal(i_tag:Longint; val:Variant);
var str0,str1:string;
    i_vstr:Integer;
begin
    str0 := InpTags.Items.Strings[i_tag];
    i_vstr := Pos(' =',str0);
    str1 := Copy(str0,0,i_vstr);
    str1 := str1 + '=' + VarToStr(val);
    InpTags.Items.Strings[i_tag] := str1;
end;

procedure TTmDelCtl.OnSend(Sender: TObject);
var t1,t2,t3,v: Variant;
    f:Single;
    i:Longint;
    s:string;
begin
    if o_tags_count > 0 then
    begin
        s := ToSend.Text;
        f := StrToFloat(s);
        v := f;
        t1 := o_chans;
        t2 := o_attrs;
        for i := 0 to o_tags_count-1 do
            o_vals[i] := v;
        t3 := o_vals;
        m_tmserver.writeValues(o_tags_count, t1, t2, t3,status);
    end;
end;

procedure TTmDelCtl.OnValClick(Sender: TObject);
begin
    ToSend.SetFocus();
end;

procedure TTmDelCtl.OnNeedAlarmClick(Sender: TObject);
begin
    if NeedAlarm.Checked then
        tfs := tfs or TF_NEEDALARM
    else
        tfs := tfs and (not TF_NEEDALARM);
    Alarm.Visible := NeedAlarm.Checked;
end;

procedure TTmDelCtl.OnStartHideClick(Sender: TObject);
begin
    if HideOnRun.Checked then
        tfs := tfs or TF_HIDEONRUN
    else
        tfs := tfs and (not TF_HIDEONRUN);
end;

procedure TTmDelCtl.OnRefreshClick(Sender: TObject);
begin
    FEvents.OnRebind();
end;

function TTmDelCtl.Get_TmxStyle: Integer;
begin
    Result := tfs;
end;

procedure TTmDelCtl.Set_TmxStyle(Value: Integer);
begin
    tfs := Value;
end;

initialization
  TActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    TTmDelCtl,
    Class_TmDelCtl,
    1,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL,
    tmApartment);
end.
