// $Id: rtmdata.h,v 1.5 2000/09/04 00:00:08 slavikb Exp $

#ifndef	_RTMDATA_H_
#define	_RTMDATA_H_

////////////////////////////////////////////////////////////////
// Data value types

#define RTM_DATATYPE_FLOAT		0
#define RTM_DATATYPE_INT		1
#define RTM_DATATYPE_STRING		2
#define RTM_DATATYPE_TIME		3

////////////////////////////////////////////////////////////////
// data quality (follows OPC classification)

#define RTM_QUALITY_MASK		0xC0
#define RTM_STATUS_MASK			0xFC
#define RTM_LIMIT_MASK			0x03

// Values for QUALITY_MASK bit field
//
#define RTM_QUALITY_BAD			0x00
#define RTM_QUALITY_UNCERTAIN		0x40
#define RTM_QUALITY_GOOD		0xC0

// STATUS_MASK Values for Quality = BAD
//
#define RTM_QUALITY_CONFIG_ERROR	0x04
#define RTM_QUALITY_NOT_CONNECTED	0x08
#define RTM_QUALITY_DEVICE_FAILURE	0x0C
#define RTM_QUALITY_SENSOR_FAILURE	0x10
#define RTM_QUALITY_LAST_KNOWN		0x14
#define RTM_QUALITY_COMM_FAILURE	0x18
#define RTM_QUALITY_OUT_OF_SERVICE	0x1C

// STATUS_MASK Values for Quality = UNCERTAIN
//
#define RTM_QUALITY_LAST_USABLE		0x44
#define RTM_QUALITY_SENSOR_CAL		0x50
#define RTM_QUALITY_EGU_EXCEEDED	0x54
#define RTM_QUALITY_SUB_NORMAL		0x58

// STATUS_MASK Values for Quality = GOOD
//
#define RTM_QUALITY_LOCAL_OVERRIDE	0xD8

// Values for Limit Bitfield 
//
#define RTM_LIMIT_OK			0x00
#define RTM_LIMIT_LOW			0x01
#define RTM_LIMIT_HIGH			0x02
#define RTM_LIMIT_CONST			0x03


////////////////////////////////////////////////////////////////
// Group definitions for object browsing

#define	RTM_GROUP_OBJECTS		0x01		// enum objects IDs
#define RTM_GROUP_CHANNELS		0x02		// enum channels IDs by object
#define RTM_GROUP_CHAN_ATTRS		0x03		// enum attributes by channel ID
#define	RTM_GROUP_CHAN_ARCH_ATTRS	0x04		// the same as previous, but only archived
#define RTM_GROUP_CAT_ATTRS		0x05		// enum attributes by category flags

////////////////////////////////////////////////////////////////
// flags for RTM_GROUP_CAT_ATTRS enumerations

#define RTM_ATTR_CAT_GENERAL		0x01		// include attributes common for analog and discrete channels
#define RTM_ATTR_CAT_ANALOG		0x02		// include attributes specific to analog channels
#define RTM_ATTR_CAT_DISCRETE		0x04		// include attributes specific to discrete channels
#define RTM_ATTR_CAT_ARCHIVED		0x08		// include only 'archived' attributes

////////////////////////////////////////////////////////////////
// attribute classes

#define RTM_ATTR_CLASS_MASK		0xFF0000L		// attribute class mask
#define RTM_ATTR_CLASS_CHANNEL		0x000000L		// channel data attributes
#define RTM_ATTR_CLASS_GLOBAL		0x010000L		// global attributes
#define RTM_ATTR_CLASS_PROJECT		0x020000L		// project attributes
#define RTM_ATTR_CLASS_NODE		0x030000L		// node attributes
#define RTM_ATTR_CLASS_OBJECT		0x040000L		// object attributes
#define RTM_ATTR_CLASS_ATTR		0x050000L		// attributes about attributes
#define	RTM_ATTR_CLASS_SESSION		0x060000L		// session-specific attributes

////////////////////////////////////////////////////////////////
// attribute class: OBJECT

#define RTM_ATTR_OBJECT_ID		0x040000L		// object ID
#define RTM_ATTR_OBJECT_NAME		0x040001L		// object name
#define RTM_ATTR_OBJECT_CHAN_CNT	0x040002L		// # of channels

////////////////////////////////////////////////////////////////
// attribute class: CHANNEL
// attribute values from 0 to 255 are reserved for legacy attributes

#define RTM_ATTR_CHANNEL_ID		0x000100L		// channel ID
#define RTM_ATTR_CHANNEL_NAME		0x000101L		// channel name
#define	RTM_ATTR_CHANNEL_UNITS		0x000102L		// units (string)
#define	RTM_ATTR_CHANNEL_PERIOD		0x000103L		// period (time)
#define	RTM_ATTR_CHANNEL_TYPE		0x000104L		// value type
#define	RTM_ATTR_CHANNEL_NBITS		0x000105L		// #bits (discrete)
#define	RTM_ATTR_CHANNEL_ATTR_CNT	0x000106L		// # of attrs

// per-attribute values: lower 8 bits are reserved for legacy attribute ID
#define	RTM_ATTR_CHANNEL_CODE		0x000200L		// encoding (0x200-0x2FF)
#define	RTM_ATTR_CHANNEL_USIGNAL	0x000300L		// unified signal (0x300-0x3FF)

////////////////////////////////////////////////////////////////
// attribute class: ATTRS

#define	RTM_ATTR_ATTR_ID		0x050000L		// attribute ID
#define	RTM_ATTR_ATTR_NAME		0x050001L		// attribute name

////////////////////////////////////////////////////////////////
// attribute class: SESSION

#define	RTM_ATTR_SESSION_SCREEN_RIGHTS	0x060001L		// screen rights mask
#define	RTM_ATTR_SESSION_T_LAST_ALARM	0x060002L		// time of the last alarm


#endif	_RTMDATA_H_

